@@ -11,14 +11,8 @@ import luigi.contrib.s3 as luigi_s3
 DATANECTAR_CONFIG_FILENAME = 'datanectar.json'
 DATANECTAR_LOG_FILENAME = 'datanectar.log'
 DATANECTAR_LOG_PATH = os.path.abspath(DATANECTAR_LOG_FILENAME)
-DATANECTAR_ARGS_FILENAME = 'luigi_args.json'
+TASK_PARAMS_FILENAME = 'task_params.json'
 
-# Setup logging
-#logger = logging.getLogger()
-#logger.setLevel(logging.DEBUG)  # TODO: Set to critical
-#file_log_handler = logging.FileHandler(DATANECTAR_LOG_PATH)
-#logger.addHandler(file_log_handler)
-#logger.addHandler(logging.StreamHandler(sys.stdout))
 
 
 def get_datanectar_config(root):
@@ -50,13 +44,18 @@ def get_task_names_in_root(root):
     return task_list
 
 
-def get_task_name(task_obj):
+def get_task_name(task_obj, endswith='_task.py'):
     """eta/extract_task.py -> 'eta/extract_task"""
-    #print(f'get_task_name({task_obj}')
-    #print('.file = {}'.format(task_obj.__module__))
-    #if isinstance(task_file_or_obj, luigi.Task):
     module_str = task_obj.__module__
-    return module_str.replace('.', '/')
+    if module_str == '__main__':
+        fpath = sys.modules[task_obj.__module__].__file__
+        parts = fpath.split('/')
+        if root:
+            rel = parts[parts.index(root):]
+            rel[-1] = rel[-1].split('.')[0]
+            return '/'.join(rel)
+    else:
+        return module_str.replace('.', '/')
 
 
 def get_task_version(task_obj):
@@ -65,7 +64,27 @@ def get_task_version(task_obj):
     return hashlib.md5(value).hexdigest()
 
 
-def get_output_dir(root, task_obj, output_type=None, save_args=True, without_version=False):
+def save_task_params_to_s3(s3_output_path, task_obj):
+    try:
+        params_dict = task_obj.to_str_params()
+        params_file_path = f'{s3_output_path}/{TASK_PARAMS_FILENAME}'
+        params_json = json.dumps(params_dict, sort_keys=True)
+        get_s3_client().put_string(params_json, params_file_path)
+    except Exception as e:
+        print(f'Error saving task params: {e}')
+
+
+def get_output_dir(
+        root, 
+        task_obj, 
+        output_type=None, 
+        save_args=True, 
+        without_version=False,
+        local_root=None
+        ) -> str:
+    """
+Directory for a given task object's output
+    """
     config = get_datanectar_config(root)
     if not output_type:
         output_type = config.get('output_type', 'local')
@@ -83,15 +102,24 @@ def get_output_dir(root, task_obj, output_type=None, save_args=True, without_ver
         output_dir = relative_path
         if save_args:
             os.makedirs(output_dir, exist_ok=True)
-            args_file_path = f'{output_dir}/{DATANECTAR_ARGS_FILENAME}'
-            args_dict = task_obj.to_str_params()
-            with open(args_file_path, 'w') as f:
-                json.dump(args_dict, f, sort_keys=True)
+            params_file_path = f'{output_dir}/{TASK_PARAMS_FILENAME}'
+            params_dict = task_obj.to_str_params()
+            with open(params_file_path, 'w') as f:
+                json.dump(params_dict, f, sort_keys=True)
+
+        if local_root:
+            return local_root + '/' + output_dir
+
+
     elif output_type == 's3':
         bucket = config.get('bucket', 'datanectar')
+        is_env = config.get('is_env', False)
+        if is_env:
+            bucket = os.getenv(bucket)
         output_dir = f's3://{bucket}/{relative_path}'
+
         if save_args:
-            print('TODO: Save args in s3')  # TODO
+            save_task_params_to_s3(output_dir, task_obj)
 
     return output_dir
 
@@ -111,6 +139,7 @@ def get_s3_client():
         aws_secret_key = dn_env_vars['DN_AWS_SECRET_ACCESS_KEY']
         s3client = luigi_s3.S3Client(aws_access_key, aws_secret_key)
         return s3client
+    return None
 
 
 def get_luigi_output_target(root, task_obj, output_type=None, filename=None, without_version=False):
@@ -123,7 +152,6 @@ def get_luigi_output_target(root, task_obj, output_type=None, filename=None, wit
         if 'output_type' in config:
             output_type = config['output_type']
         else:
-            #logger.debug(f'(Root={root}) No output_type specified - using default: "local"')
             output_type = 'local'
 
     if output_type == 'local':
